/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

// This is an implementation of opl3 instruments for General Midi
// which tries to emulate old Creative's sbmidi.exe

#include <stdio.h>
#include <string.h>

#include "mid.h"
#include "mid_core.h"
#include "mid_drv.h"

// forward def
extern unsigned char mid_drum_notes[];
extern unsigned char mid_patches[];
extern unsigned char mid_drum_patches[];

typedef union
{
	struct
	{
		unsigned char lo;
		unsigned char hi;
	};
	struct
	{
		unsigned short freq : 10;
		unsigned short octave : 3;
		unsigned short noteon : 1;
	};
} A0B0;

typedef struct {
	unsigned char Flag; // 0 - note off, 1 - note on
	unsigned char Channel;
	unsigned char Note;
	unsigned char Volume;
	A0B0 fData;
} OPLVOICE;

unsigned char fm_drum_BD;
unsigned char fm_perc_mode; // 0 - melo, 1 - perc
// same order as drumvoices - 6
unsigned char fm_drum_fl[] = {16, 8, 4, 2, 1};
unsigned char fm_drum_voice[] = {6, 7, 8, 8, 7};
unsigned char fm_drum_offset[] = {0x10, 0x14, 0x12, 0x15, 0x11};

unsigned char fm_feedback[18]; // bit 1 - algo
unsigned char fm_keyscale[18]; // upper 2 bits of volume
unsigned char fm_level[18]; // instrument volume (63-reg43)

unsigned char fm_drum_level[5]; // drum volumes 6,7,8,9,10
unsigned char fm_drum_keyscale[5]; // upper 2 bits of volume

OPLVOICE fm_voices[18];

// register offsets (doubled for 18 channels)
int fm_offset[18] = {
	0x00, 0x01, 0x02, 0x08, 0x09, 0x0a, 0x10, 0x11, 0x12,
	0x100, 0x101, 0x102, 0x108, 0x109, 0x10a, 0x110, 0x111, 0x112
};

// register offsets (type 2)
int fm_v_offset[18] = {0,1,2,3,4,5,6,7,8,0x100,0x101,0x102,0x103,0x104,0x105,0x106,0x107,0x108};
int fm_note_freq[12] = {0x16b, 0x181, 0x198, 0x1b0, 0x1ca, 0x1e5, 0x202, 0x220, 0x241, 0x263, 0x287, 0x2ae};

/* logarithmic relationship between midi and FM volumes */
int fm_vol_table[128] = {
	0,  11, 16, 19, 22, 25, 27, 29, 32, 33, 35, 37, 39, 40, 42, 43,
	45, 46, 48, 49, 50, 51, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62,
	64, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 75, 76, 77,
	78, 79, 80, 80, 81, 82, 83, 83, 84, 85, 86, 86, 87, 88, 89, 89,
	90, 91, 91, 92, 93, 93, 94, 95, 96, 96, 97, 97, 98, 99, 99, 100,
	101, 101, 102, 103, 103, 104, 104, 105, 106, 106, 107, 107, 108,
	109, 109, 110, 110, 111, 112, 112, 113, 113, 114, 114, 115, 115,
	116, 117, 117, 118, 118, 119, 119, 120, 120, 121, 121, 122, 122,
	123, 123, 124, 124, 125, 125, 126, 126, 127
};

// voice - opl3 voice (0-17)
// vol - midi velocity value
void fm_set_volume(int voice, int vol)
{
	vol = fm_level[voice] * fm_vol_table[vol] / 128;

	fm_write(fm_offset[voice] + 0x43, (63 - vol) | fm_keyscale[voice]);

	if((fm_feedback[voice] & 1) == 1)
		fm_write(fm_offset[voice] + 0x40, (63 - vol) | fm_keyscale[voice]);
}

// 6 - bass, 7 - snare, 8 - tom, 9 - cymbal, 10 - hihat
// which map to voices 6, 7, 8, 8, 7
void fm_set_drum_volume(int drumvoice, int volume)
{
	if(drumvoice == 6) {
		fm_set_volume(6, volume);
		return;
	}

	volume = fm_drum_level[drumvoice-6] * fm_vol_table[volume] / 128;
	fm_write(fm_drum_offset[drumvoice-6] + 0x40, (63 - volume) | fm_drum_keyscale[drumvoice-6]);
}

void mid_SetChannelVolume(int channel, int volume)
{
	// midi has all drums on channel 9
	if(mid_Mode == MODE_MID && channel == 9) {
		for(int i = 6; i <= 10; i++) {
			fm_set_drum_volume(i, fm_drum_level[i-6] * volume / 128);
		}
		return;
	}

	// mdi has drums on 6,7,8,9,10
	if(mid_Mode == MODE_MDI && fm_perc_mode == 1 && channel >= 6) {
		fm_set_drum_volume(channel, fm_drum_level[channel-6] * volume / 128);

		return;
	}

	// cmf has drums on 11,12,13,14,15
	if(mid_Mode == MODE_CMF && fm_perc_mode == 1 && channel >= 11) {
		fm_set_drum_volume(channel, fm_drum_level[channel-6-5] * volume / 128);

		return;
	}

	// roll thru voices, find all channels, remember last velos and set volume
	for(int i = 0; i < 18; (fm_perc_mode == 1 ? (i == 5 ? i = 9 : i++) : i++)) {
		if(fm_voices[i].Flag == 1 && fm_voices[i].Channel == channel) {
			// channel volume * last note volume / 128
			fm_set_volume(i, fm_voices[i].Volume * volume / 128 );
		}
	}
}

void fm_set_drum_trig(int drumvoice, int flag)
{
	fm_drum_BD &= ~fm_drum_fl[drumvoice-6];
	if(flag != 0) fm_drum_BD |= fm_drum_fl[drumvoice-6];
	fm_write(0xbd, fm_drum_BD);
}

void fm_load_mid_patch(int voice, unsigned char *p)
{
	int r;

	fm_keyscale[voice] = *(p + 3) & 0xc0; // save upper 2 bits of vol
	fm_level[voice] = 63 - (*(p + 3) & 0x3f);
	fm_feedback[voice] = *(p + 10) | 0x30; // save feedback + algo + stereo

	r = fm_offset[voice];

	fm_write(r + 0x20, *(p + 0));
	fm_write(r + 0x23, *(p + 1));

	fm_write(r + 0x40, *(p + 2));
	fm_write(r + 0x43, *(p + 3));

	fm_write(r + 0x60, *(p + 4));
	fm_write(r + 0x63, *(p + 5));

	fm_write(r + 0x80, *(p + 6));
	fm_write(r + 0x83, *(p + 7));

	fm_write(r + 0xe0, *(p + 8));
	fm_write(r + 0xe3, *(p + 9));

	fm_write(fm_v_offset[voice] + 0xc0, *(p + 10) | 0x30); // stereo
}

// 6 - bass, 7 - snare, 8 - tom, 9 - cymbal, 10 - hihat
// which map to voices 6, 7, 8, 8, 7
void fm_load_drum_patch(int drumvoice, unsigned char *p)
{
	int r;

	// bass load as ordinary
	if(drumvoice == 6) {
		fm_load_mid_patch(6, p);
		return;
	}

	r = fm_drum_offset[drumvoice-6];

	fm_drum_keyscale[drumvoice-6] = *(p + 2) & 0xc0; // save upper 2 bits of vol
	fm_drum_level[drumvoice-6] = 63 - (*(p + 2) & 0x3f);

	fm_write(r + 0x20, *(p + 0));
	fm_write(r + 0x40, *(p + 2));
	fm_write(r + 0x60, *(p + 4));
	fm_write(r + 0x80, *(p + 6));
	fm_write(r + 0xe0, *(p + 8));

	//if(drumvoice < 9)
	fm_write(fm_v_offset[fm_drum_voice[drumvoice-6]] + 0xc0, (*(p + 10) & 0x3F) | 0x30); // stereo
}

// finds voice
int fm_find_voice(int channel, int note)
{
	// if mdi, channel == voice
	if(mid_Mode == MODE_MDI) return channel;

	for(int i = 0; i < 18; (fm_perc_mode == 1 ? (i == 5 ? i = 9 : i++) : i++)) {
		if(fm_voices[i].Flag == 1 && fm_voices[i].Channel == channel && fm_voices[i].Note == note) return i;
	}

	return -1;
}

// rolls thru fm_voices and returns voice which has Flag==0
int fm_get_free_voice(int channel, int note)
{
	// if mdi, channel == voice
	if(mid_Mode == MODE_MDI) return channel;

	for(int i = 0; i < 18; (fm_perc_mode == 1 ? (i == 5 ? i = 9 : i++) : i++))
		if(fm_voices[i].Flag == 0) {
			/*for(int j = i; j < 18; (j == 5 ? j = 9 : j++))
				if(fm_voices[j].Flag == 0) return j;*/
			return i;
		}

	for(int i = 0; i < 18; (fm_perc_mode == 1 ? (i == 5 ? i = 9 : i++) : i++))
		if(fm_voices[i].Flag == 1 && fm_voices[i].Channel == channel) {
			return i;
		}

	return -1;
}

void mid_NoteOff(unsigned char channel, unsigned char note, unsigned char volume)
{
	int voice;

	// if normal mid
	if(mid_Mode == MODE_MID && channel == 9) { // if normal midi
		int drumvoice;

		// if out of range
		if(note < 35 || note > 81) return;

		drumvoice = mid_drum_patches[(note-35)*16 + 11];
		// shut voice if it wasn't off
		fm_set_drum_trig(drumvoice, 0);

		return;
	}

	if(fm_perc_mode == 1) {
		// mdi - drum voices 6,7,8,9,10
		if(mid_Mode == MODE_MDI && channel >= 6) {
			fm_set_drum_trig(channel, 0);
			return;
		}

		// cmf - drum voices 11,12,13,14,15
		if(mid_Mode == MODE_CMF && channel >= 11) {
			fm_set_drum_trig(channel-5, 0);
			return;
		}
	}

	// find the exact note we need to shut up
	voice = fm_find_voice(channel, note);

	if(voice == -1) {
		FPRINTF(stdout, "NoteOff failed\n");
		return;
	}

	fm_voices[voice].Flag = 0;
	fm_voices[voice].fData.noteon = 0;

	fm_write(0xa0 + fm_v_offset[voice], fm_voices[voice].fData.lo);
	fm_write(0xb0 + fm_v_offset[voice], fm_voices[voice].fData.hi);

	//fprintf(stdout, "NOTEOFF: FM: Voice: %i, Note: %i, Volume: %i\n", channel, note, volume);
}

void fm_mid_drum_on(unsigned char channel, unsigned char note, unsigned char volume)
{
	int drumvoice;

	// if out of range
	if(note < 35 || note > 81) return;
	drumvoice = mid_drum_patches[(note-35)*16 + 11];

	// shut voice if it wasn't off
	fm_set_drum_trig(drumvoice, 0);
	fm_set_drum_volume(drumvoice, 0);
	fm_load_drum_patch(drumvoice, &mid_drum_patches[(note-35)*16]);

	if(drumvoice == 6 || drumvoice == 8 /*|| drumvoice == 9*/) { // if 6 or 8 WHY NO SNARE DRUM HERE??
		fm_voices[drumvoice].fData.freq = fm_note_freq[mid_drum_notes[note-35] % 12];
		fm_voices[drumvoice].fData.octave = mid_drum_notes[note-35] / 12;

		fm_voices[drumvoice].fData.noteon = 0;
		fm_write(0xa0 + drumvoice, fm_voices[drumvoice].fData.lo);
		fm_write(0xb0 + drumvoice, fm_voices[drumvoice].fData.hi);
	}

	// note on
	fm_set_drum_volume(drumvoice, MidiChannels[channel].Volume * volume / 128);
	fm_set_drum_trig(drumvoice, 1);
	return;
}

void fm_mdi_drum_on(unsigned char drumvoice, unsigned char note, unsigned char volume)
{
	// shut voice if it wasn't off
	fm_set_drum_trig(drumvoice, 0);

	if(drumvoice < 9) { // if 6, 7 or 8
		fm_voices[drumvoice].fData.freq = fm_note_freq[(drumvoice == 6 ? 0 : 7)];
		fm_voices[drumvoice].fData.octave = 2;
		fm_voices[drumvoice].fData.noteon = 0; // 0 obligatory
		fm_write(0xa0 + drumvoice, fm_voices[drumvoice].fData.lo);
		fm_write(0xb0 + drumvoice, fm_voices[drumvoice].fData.hi);
	}

	// note on
	fm_set_drum_volume(drumvoice, MidiChannels[drumvoice].Volume * volume / 128);
	fm_set_drum_trig(drumvoice, 1);
}

void fm_set_note(unsigned char voice, unsigned char note, int pitchbend, int pitchrange)
{
	// this table holds frequency differences between notes
	static int logtable[12] = {20, 21, 23, 24, 26, 27, 29, 30, 33, 34, 37, 40};
	int pitchvalue, absnote, newnote;

	// make pitchbend a signed value
	if(pitchbend > 0x2000) pitchbend -= 0x2000;
	else pitchbend = 0 - (0x2000 - pitchbend);

	// take pitch range into account
	pitchvalue = pitchbend * pitchrange / 0x2000;

	// calculate "absolute note" and +/- pitchbend value
	absnote = ((note - 12) << 7) + pitchvalue;

	// cut off too low and too high values
	if(absnote < 0) absnote = 0;
	if(absnote >= 0x2f80) absnote = 0x2f7f; // 2f80

	// calculate the new note
	newnote = (absnote % (12 << 7)) >> 7;

	// calculate frequency + pitchbend cents recalc'ed into logarithmic values
	fm_voices[voice].fData.freq = fm_note_freq[newnote] + (absnote & 0x7f) * logtable[newnote] / 128;
	fm_voices[voice].fData.octave = absnote / (12 << 7);

	fm_voices[voice].fData.noteon = 1;

	fm_write(0xa0 + fm_v_offset[voice], fm_voices[voice].fData.lo);
	fm_write(0xb0 + fm_v_offset[voice], fm_voices[voice].fData.hi);
}

void mid_SetChannelPitchWheel(int channel)
{
	// roll thru voices, find all channels, remember last velos and set volume
	for(int i = 0; i < 18; (fm_perc_mode == 1 ? (i == 5 ? i = 9 : i++) : i++)) {
		if(fm_voices[i].Flag == 1 && fm_voices[i].Channel == channel) {
			fm_set_note(i, fm_voices[i].Note,
				MidiChannels[channel].PitchWheel, MidiChannels[channel].PitchRange);
		}
	}
}

void mid_NoteOn(unsigned char channel, unsigned char note, unsigned char volume)
{
	int voice;

	// if normal midi, percussion is always on
	if(mid_Mode == MODE_MID && channel == 9) { fm_mid_drum_on(channel, note, volume); return; }

	if(fm_perc_mode == 1) {
		// mdi - drum voices 6,7,8,9,10
		if(mid_Mode == MODE_MDI && channel >= 6) {
			fm_mdi_drum_on(channel, note, volume);
			return;
		}

		// cmf - drum voices 11,12,13,14,15
		if(mid_Mode == MODE_CMF && channel >= 11) {
			fm_mdi_drum_on(channel-5, note, volume);
			return;
		}
	}

	if(volume == 0) { mid_NoteOff(channel, note, 0); return; }

	// MID and CMF are polyphonic, voice is dynamic
	// MDI - monophonic, voice == channel (except for drums)
	voice = fm_get_free_voice(channel, note);
	if(voice == -1) {
		FPRINTF(stdout, "fm_get_free_voice failed\n");
		return;
	}

	fm_voices[voice].fData.noteon = 0;
	fm_write(0xb0 + fm_v_offset[voice], fm_voices[voice].fData.hi);

	fm_voices[voice].Flag = 1;
	fm_voices[voice].Channel = channel;
	fm_voices[voice].Note = note;
	fm_voices[voice].Volume = volume;

	// MID and CMF load patches dynamically
	// MDI - only by command
	if(mid_Mode == MODE_MID) fm_load_mid_patch(voice, &mid_patches[MidiChannels[channel].Patch*16]);
	/*else if(mid_Mode == MODE_CMF) fm_load_cmf_patch(voice, ... ); */

	// channel volume * note volume / 128
	fm_set_volume(voice, MidiChannels[channel].Volume * volume / 128);

	// set note with regard to pitchwheel and pitch sensivity
	fm_set_note(voice, note, MidiChannels[channel].PitchWheel, MidiChannels[channel].PitchRange);
}

#pragma pack(push)
#pragma pack(1)

typedef struct {
	struct {
		unsigned char KeyScaleLevel;	// 0
		unsigned char FrequencyMult;	// 1
		unsigned char Feedback;		// 2
		unsigned char AttackRate;	// 3
		unsigned char SustainLevel;	// 4
		unsigned char Sustain;		// 5
		unsigned char DecayRate;	// 6
		unsigned char ReleaseRate;	// 7
		unsigned char OutputLevel;	// 8
		unsigned char AmplitudeVibrato;	// 9
		unsigned char FrequencyVibrato;	// 10
		unsigned char EnvelopeScaling;	// 11
		unsigned char FmType;		// 12
	} op[2];

	unsigned char WaveForm[2];		// 26, 27
} MDI_TIMBRE;

typedef struct {
	struct {
		unsigned char FrequencyMult : 4;
		unsigned char KeyboardScalingRate : 1;
		unsigned char Sustain : 1;
		unsigned char FrequencyVibrato : 1;
		unsigned char AmplitudeVibrato : 1;
	} r20[2];
	struct {
		unsigned char OutputLevel : 6;
		unsigned char ScalingLevel : 2;
	} r40[2];
	struct {
		unsigned char DecayRate : 4;
		unsigned char AttackRate : 4;
	} r60[2];
	struct {
		unsigned char ReleaseRate : 4;
		unsigned char SustainLevel : 4;
	} r80[2];
	struct {
		unsigned char WaveForm : 3;
	} re0[2];
	struct {
		unsigned char Algorithm : 1;
		unsigned char Feedback : 3;
		unsigned char Stereo : 2;
	} rc0;
} TIMBRE;

#pragma pack(pop)

void mid_MdiPatchLoad(unsigned char channel, unsigned char *p)
{
	static TIMBRE dest;
	MDI_TIMBRE *pSrc = (MDI_TIMBRE *)p;

	// convert from .BNK format
	for(int i = 0; i <= 1; i++) {
		dest.r20[i].FrequencyMult = pSrc->op[i].FrequencyMult;
		dest.r20[i].KeyboardScalingRate = pSrc->op[i].EnvelopeScaling;
		dest.r20[i].Sustain = pSrc->op[i].Sustain;
		dest.r20[i].FrequencyVibrato = pSrc->op[i].FrequencyVibrato;
		dest.r20[i].AmplitudeVibrato = pSrc->op[i].AmplitudeVibrato;

		dest.r40[i].OutputLevel = pSrc->op[i].OutputLevel;
		dest.r40[i].ScalingLevel = pSrc->op[i].KeyScaleLevel;

		dest.r60[i].DecayRate = pSrc->op[i].DecayRate;
		dest.r60[i].AttackRate = pSrc->op[i].AttackRate;

		dest.r80[i].ReleaseRate = pSrc->op[i].ReleaseRate;
		dest.r80[i].SustainLevel = pSrc->op[i].SustainLevel;

		dest.re0[i].WaveForm = pSrc->WaveForm[i];
	}

	dest.rc0.Algorithm = pSrc->op[0].FmType ^ 1; // Was it inverted?? WTF??
	dest.rc0.Feedback = pSrc->op[0].Feedback;
	dest.rc0.Stereo = 3;

	// in perc mode voices 6,7,8,9,10 are percussive, 9 maps to 8, 10 maps to 7
	if(fm_perc_mode == 1 && channel >= 6) {
		fm_load_drum_patch(channel, (unsigned char *)&dest);
		return;
	}

	fm_load_mid_patch(channel, (unsigned char *)&dest);
}

void mid_MdiSetPercMode(unsigned char m)
{
	fm_perc_mode = (m & 1);
	fm_drum_BD = 0xc0 | ((m & 1) << 5);
	fm_write(0xBD, fm_drum_BD);
}

// reset all voices
void mid_Reset()
{
	// proper reset for mid, mdi, cmf
	if(mid_Mode == MODE_MID) { // normal mid
		fm_perc_mode = 1;
		fm_drum_BD = 0xe0;
	} else { // adlib mdi (1) or cmf (2)
		fm_perc_mode = 0;
		fm_drum_BD = 0xc0;
	}

	fm_write(0xBD, fm_drum_BD);

	// all notes off
	for(int i = 0; i < 18; i++) {
		fm_voices[i].Flag = 0;
		fm_voices[i].fData.noteon = 0;
		fm_write(0xb0 + fm_v_offset[i], fm_voices[i].fData.hi);
		fm_set_volume(i, 0);
	}
}

unsigned char mid_drum_notes[] =
{
	0x2F, 0x24, 0x43, 0x3C, 0x3C, 0x3C, 0x30, 0x3C, 0x34, 0x3C, 0x37,
	0x3C, 0x3C, 0x40, 0x3C, 0x43, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x30,
	0x3C, 0x3C, 0x3C, 0x43, 0x3E, 0x43, 0x43, 0x3C, 0x3C, 0x37, 0x35,
	0x30, 0x3C, 0x3C, 0x4F, 0x4F, 0x3C, 0x3C, 0x5B, 0x3C, 0x35, 0x3C,
	0x3C, 0x4F, 0x4F, 0
};

// format supposedly 20, 23, 40, 43, 60, 63, 80, 83, E0, E3, C0
unsigned char mid_patches[] =
{
	0x01, 0x11, 0x4F, 0x00, 0xF1, 0xD2, 0x51, 0x43, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x02, 0x12, 0x4F, 0x06, 0xF1, 0xD2, 0x51, 0x43, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x11, 0x4A, 0x04, 0xF1, 0xD2, 0x53, 0x74, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x03, 0x11, 0x4F, 0x04, 0xF1, 0xD2, 0x53, 0x74, 0x01, 0x01, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x01, 0x11, 0x66, 0x00, 0xF1, 0xD2, 0x51, 0xC3, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xC0, 0xD2, 0x52, 0x05, 0xF1, 0xD2, 0x53, 0x94, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x12, 0x18, 0x86, 0x00, 0xF3, 0xFC, 0x00, 0x33, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xD0, 0x12, 0x4E, 0x03, 0xA8, 0x92, 0x32, 0xA7, 0x03, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xC8, 0xD1, 0x4F, 0x08, 0xF2, 0xF3, 0x64, 0x77, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x33, 0x34, 0x0E, 0x00, 0x01, 0x7D, 0x11, 0x34, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x17, 0x16, 0x50, 0x00, 0xD1, 0xD3, 0x52, 0x92, 0x00, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE7, 0xE1, 0x21, 0x06, 0xF5, 0xF6, 0x77, 0x14, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x95, 0x81, 0x4E, 0x00, 0xDA, 0xF9, 0x25, 0x15, 0x00, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x27, 0x21, 0x1F, 0x03, 0xF5, 0xF5, 0x96, 0x57, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x87, 0xF1, 0x4E, 0x80, 0xB1, 0xE6, 0x33, 0x42, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x31, 0x11, 0x87, 0x80, 0xA1, 0x7D, 0x11, 0x43, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x32, 0xB1, 0x8C, 0x03, 0x91, 0xA1, 0x07, 0x19, 0x02, 0x00, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x31, 0xB4, 0x54, 0x83, 0xF1, 0xF5, 0x07, 0x19, 0x00, 0x00, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x24, 0x21, 0x40, 0x53, 0xFF, 0xFF, 0x0F, 0x0F, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xD2, 0xF1, 0x44, 0x80, 0x91, 0xA1, 0x57, 0x09, 0x01, 0x01, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x01, 0x02, 0x52, 0x88, 0xF0, 0xF0, 0x1F, 0x1F, 0x01, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x21, 0x32, 0x4F, 0x0B, 0xF2, 0x52, 0x0B, 0x0B, 0x00, 0x01, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xF0, 0xF2, 0x93, 0x07, 0xD8, 0xB3, 0x0B, 0x0B, 0x02, 0x01, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x20, 0x31, 0x5D, 0x07, 0xF2, 0x52, 0x0B, 0x0B, 0x03, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x01, 0x01, 0x1B, 0x04, 0xF4, 0xF3, 0x25, 0x46, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x11, 0x01, 0x0F, 0x07, 0xF4, 0xF3, 0x25, 0x46, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x01, 0x01, 0x27, 0x00, 0xF1, 0xF4, 0x1F, 0x88, 0x02, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x12, 0x13, 0x44, 0x03, 0xEA, 0xD2, 0x32, 0xE7, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x30, 0x31, 0x45, 0x00, 0xA4, 0xF5, 0x32, 0xE7, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x21, 0x21, 0x0F, 0x03, 0xF5, 0xF1, 0x17, 0x78, 0x02, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x01, 0x20, 0x41, 0x07, 0xD1, 0xC1, 0x34, 0xA5, 0x03, 0x03, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x10, 0x12, 0x43, 0x02, 0xA7, 0xE3, 0x97, 0xE7, 0x03, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x20, 0x21, 0x28, 0x01, 0xC5, 0xD2, 0x15, 0xA4, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x30, 0x21, 0x16, 0x05, 0xF2, 0xF3, 0x9F, 0x78, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x30, 0x21, 0x11, 0x05, 0x82, 0xF3, 0x9F, 0x78, 0x00, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x21, 0x21, 0x23, 0x00, 0x73, 0x93, 0x1A, 0x87, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x30, 0x21, 0x0E, 0x09, 0x62, 0xF3, 0x55, 0x68, 0x02, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x30, 0x22, 0x0C, 0x00, 0x62, 0xD5, 0xB5, 0x98, 0x01, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x70, 0x72, 0x93, 0x43, 0x64, 0xA1, 0x43, 0x43, 0x00, 0x00, 0x0A, 0x06, 0x00, 0x00, 0x00, 0x00,
	0x30, 0x32, 0x8D, 0x85, 0x44, 0x92, 0x43, 0x43, 0x02, 0x00, 0x0A, 0x06, 0x00, 0x00, 0x00, 0x00,
	0xE1, 0xE2, 0x4E, 0x00, 0x65, 0x61, 0x43, 0x44, 0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xA1, 0xA2, 0x8E, 0x05, 0x65, 0x63, 0x43, 0x45, 0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xB0, 0x61, 0x87, 0x40, 0xD1, 0x62, 0x11, 0x15, 0x02, 0x01, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xF0, 0x20, 0x8A, 0x80, 0xB1, 0xA0, 0x11, 0x15, 0x02, 0x01, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xF1, 0xE2, 0x89, 0x43, 0x73, 0x43, 0x01, 0x05, 0x02, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x31, 0x21, 0x57, 0x80, 0xF8, 0xF7, 0xF9, 0xE6, 0x03, 0x02, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x32, 0x01, 0x24, 0x82, 0xF1, 0xF5, 0x35, 0x35, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x04, 0x00, 0xAA, 0xD2, 0xC8, 0xB3, 0x00, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE0, 0xF1, 0x4F, 0x03, 0xD4, 0x55, 0x0B, 0x0B, 0x02, 0x02, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE0, 0xF0, 0x52, 0x00, 0x96, 0x35, 0x05, 0x01, 0x02, 0x02, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE1, 0xF1, 0x4F, 0x00, 0x36, 0x45, 0x05, 0x02, 0x02, 0x02, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE2, 0xE1, 0x48, 0x81, 0x21, 0x41, 0x43, 0x45, 0x02, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE0, 0xF1, 0x16, 0x00, 0x41, 0x20, 0x52, 0x72, 0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE0, 0xF1, 0x11, 0x00, 0x01, 0xD0, 0x52, 0x72, 0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE0, 0xF1, 0x1A, 0x00, 0x61, 0x30, 0x52, 0x73, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x50, 0x50, 0x0B, 0x07, 0x84, 0xA4, 0x4B, 0x99, 0x00, 0x00, 0x0A, 0x06, 0x00, 0x00, 0x00, 0x00,
	0x31, 0x61, 0x1C, 0x84, 0x41, 0x92, 0x0B, 0x3B, 0x00, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xB1, 0x61, 0x1C, 0x05, 0x41, 0x92, 0x1F, 0x3B, 0x00, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x20, 0x21, 0x18, 0x00, 0x52, 0xA2, 0x15, 0x24, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xC1, 0xC1, 0x94, 0x84, 0x74, 0xA3, 0xEA, 0xF5, 0x02, 0x01, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x21, 0x21, 0x28, 0x00, 0x41, 0x81, 0xB4, 0x98, 0x00, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x21, 0x21, 0x1D, 0x01, 0x51, 0xE1, 0xAE, 0x3E, 0x02, 0x01, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE0, 0xE0, 0x93, 0x82, 0x51, 0x81, 0xA6, 0x97, 0x02, 0x01, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE0, 0xE1, 0x93, 0x83, 0x51, 0xE1, 0xA6, 0x97, 0x02, 0x01, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE0, 0xF2, 0x4B, 0x0B, 0xD8, 0xB3, 0x0B, 0x0B, 0x02, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE0, 0xF1, 0x49, 0x0B, 0xB8, 0xB3, 0x0B, 0x0B, 0x02, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE0, 0xF0, 0x4E, 0x0B, 0x98, 0xC3, 0x0B, 0x0B, 0x01, 0x02, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE0, 0xF1, 0x4C, 0x0B, 0x88, 0xD3, 0x0B, 0x0B, 0x01, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xF1, 0xE4, 0xC5, 0x08, 0x7E, 0x8C, 0x17, 0x0E, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x60, 0x72, 0x4F, 0x0A, 0xD8, 0xB3, 0x0B, 0x0B, 0x00, 0x01, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x31, 0x72, 0xD1, 0x80, 0xD5, 0x91, 0x19, 0x1B, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x32, 0x71, 0xC8, 0x80, 0xD5, 0x73, 0x19, 0x1B, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE2, 0x62, 0x6A, 0x00, 0x9E, 0x55, 0x8F, 0x2A, 0x00, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE0, 0x61, 0xEC, 0x00, 0x7E, 0x65, 0x8F, 0x2A, 0x00, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x62, 0xA2, 0x88, 0x8D, 0x84, 0x75, 0x27, 0x17, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x62, 0xA2, 0x84, 0x8D, 0x84, 0x75, 0x27, 0x17, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE3, 0x62, 0x6D, 0x00, 0x57, 0x57, 0x04, 0x77, 0x00, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xF1, 0xE1, 0x28, 0x00, 0x57, 0x67, 0x34, 0x5D, 0x03, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xD1, 0x72, 0xC7, 0x03, 0x31, 0x42, 0x0F, 0x09, 0x00, 0x00, 0x0B, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xF2, 0x72, 0xC7, 0x05, 0x51, 0x42, 0x05, 0x69, 0x00, 0x00, 0x0B, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x23, 0x31, 0x4F, 0x06, 0x51, 0x60, 0x5B, 0x25, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x22, 0x31, 0x48, 0x06, 0x31, 0xC0, 0x9B, 0x65, 0x02, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xF1, 0xE1, 0x28, 0x04, 0x57, 0x67, 0x34, 0x0D, 0x03, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE1, 0xE1, 0x23, 0x00, 0x57, 0x67, 0x04, 0x4D, 0x03, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE2, 0x31, 0x42, 0x12, 0x78, 0xF3, 0x0B, 0x0B, 0x01, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE2, 0xE2, 0x21, 0x07, 0x11, 0x40, 0x52, 0x73, 0x01, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x23, 0xA4, 0xC0, 0x00, 0x51, 0x35, 0x07, 0x79, 0x01, 0x02, 0x0D, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x24, 0xA0, 0xC0, 0x01, 0x51, 0x75, 0x07, 0x09, 0x01, 0x02, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE0, 0xF0, 0x16, 0x03, 0xB1, 0xE0, 0x51, 0x75, 0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x03, 0xA4, 0xC0, 0x04, 0x52, 0xF4, 0x03, 0x55, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE1, 0xE1, 0x93, 0x81, 0x31, 0xA1, 0xA6, 0x97, 0x01, 0x01, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xF0, 0x71, 0xC4, 0x87, 0x10, 0x11, 0x01, 0xC1, 0x02, 0x02, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xC1, 0xE0, 0x4F, 0x00, 0xB1, 0x12, 0x53, 0x74, 0x02, 0x02, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xC0, 0x41, 0x6D, 0x07, 0xF9, 0xF2, 0x21, 0xB3, 0x01, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE3, 0xE2, 0x4C, 0x07, 0x21, 0xA1, 0x43, 0x45, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE3, 0xE2, 0x0C, 0x09, 0x11, 0x80, 0x52, 0x73, 0x01, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x26, 0x88, 0xC0, 0x00, 0x55, 0xF8, 0x47, 0x19, 0x00, 0x00, 0x0B, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x23, 0xE4, 0xD4, 0x00, 0xE5, 0x35, 0x03, 0x65, 0x00, 0x00, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x27, 0x32, 0xC0, 0x07, 0x32, 0xA4, 0x62, 0x33, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xD0, 0x31, 0x4E, 0x03, 0x98, 0xA2, 0x32, 0x47, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xF0, 0x71, 0xC0, 0x04, 0x93, 0x43, 0x03, 0x02, 0x01, 0x00, 0x0F, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE0, 0xF1, 0x1A, 0x82, 0x13, 0x33, 0x52, 0x13, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE0, 0xF1, 0x1A, 0x04, 0x45, 0x32, 0xBA, 0x91, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x11, 0x15, 0x18, 0x0D, 0x58, 0xA2, 0x02, 0x72, 0x01, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x10, 0x18, 0x80, 0x45, 0xF1, 0xF1, 0x53, 0x53, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x31, 0x17, 0x86, 0x80, 0xA1, 0x7D, 0x11, 0x23, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x10, 0x18, 0x80, 0x40, 0xF1, 0xF6, 0x53, 0x54, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x31, 0x34, 0x21, 0x02, 0xF5, 0x93, 0x56, 0xE8, 0x01, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x03, 0x15, 0x4F, 0x03, 0xF1, 0xD6, 0x39, 0x74, 0x03, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x31, 0x22, 0x43, 0x06, 0x6E, 0x8B, 0x17, 0x0C, 0x01, 0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x31, 0x22, 0x1C, 0x89, 0x61, 0x52, 0x03, 0x67, 0x00, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x60, 0xF0, 0x0C, 0x89, 0x81, 0x61, 0x03, 0x0C, 0x00, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x27, 0x05, 0x55, 0x05, 0x31, 0xA7, 0x62, 0x75, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x95, 0x16, 0x81, 0x00, 0xE7, 0x96, 0x01, 0x67, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x0C, 0x01, 0x87, 0x80, 0xF0, 0xF2, 0x05, 0x05, 0x01, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x35, 0x11, 0x44, 0x00, 0xF8, 0xF5, 0xFF, 0x75, 0x00, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x10, 0x10, 0x0B, 0x08, 0xA7, 0xD5, 0xEC, 0xF5, 0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00,
	0x20, 0x01, 0x0B, 0x07, 0xA8, 0xD6, 0xC8, 0xB7, 0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x01, 0x0B, 0x00, 0x88, 0xD5, 0xC4, 0xB7, 0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00,
	0x0C, 0x10, 0x8F, 0x80, 0x41, 0x33, 0x31, 0x2B, 0x00, 0x03, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x17, 0xF7, 0x00, 0x00, 0x3B, 0xEA, 0xDF, 0x97, 0x03, 0x00, 0x0B, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x12, 0x18, 0x06, 0x09, 0x73, 0x3C, 0x02, 0x74, 0x00, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x02, 0x08, 0x00, 0x02, 0x3E, 0x14, 0x01, 0xF3, 0x02, 0x02, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xF5, 0xF6, 0xD4, 0x00, 0xEB, 0x45, 0x03, 0x68, 0x00, 0x00, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xF0, 0xCA, 0x00, 0xC0, 0xDA, 0xB0, 0x71, 0x17, 0x01, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xF0, 0xE2, 0x00, 0xC0, 0x1E, 0x11, 0x11, 0x11, 0x01, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xE7, 0xE8, 0x00, 0x0E, 0x34, 0x10, 0x00, 0xB2, 0x02, 0x02, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x0C, 0x04, 0x00, 0x00, 0xF0, 0xF6, 0xF0, 0xE6, 0x02, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00
};

unsigned char mid_drum_patches[] = {
	0x00, 0x00, 0x0B, 0x00, 0xA8, 0xD6, 0x4C, 0x45, 0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x0B, 0x00, 0xAA, 0xD2, 0xC8, 0xB7, 0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00,
	0x26, 0x00, 0x00, 0x00, 0xF0, 0xFA, 0xF0, 0xB7, 0x03, 0x03, 0x0E, 0x06, 0x00, 0x00, 0x00, 0x00,
	0x10, 0xC2, 0x07, 0x23, 0xF7, 0xE0, 0xF5, 0x41, 0x02, 0x02, 0x82, 0x07, 0x00, 0x00, 0x00, 0x00,
	0xF2, 0xF1, 0x0A, 0x38, 0x88, 0xAD, 0xF4, 0x88, 0x02, 0x02, 0x02, 0x07, 0x00, 0x00, 0x00, 0x00,
	0xD0, 0xC2, 0x81, 0x23, 0xA6, 0xE0, 0xF6, 0x41, 0x02, 0x02, 0x81, 0x07, 0x00, 0x00, 0x00, 0x00,
	0x40, 0xC2, 0x00, 0x23, 0xF5, 0xE0, 0x38, 0x41, 0x00, 0x02, 0x05, 0x08, 0x00, 0x00, 0x00, 0x00,
	0x01, 0xC2, 0x03, 0x23, 0xB8, 0xE0, 0xB5, 0x41, 0x01, 0x02, 0x7D, 0x0A, 0x00, 0x00, 0x00, 0x00,
	0x40, 0xC2, 0x00, 0x23, 0xF5, 0xE0, 0x38, 0x41, 0x00, 0x02, 0xF1, 0x08, 0x00, 0x00, 0x00, 0x00,
	0x01, 0xB3, 0x08, 0xC1, 0x88, 0x18, 0xA5, 0x50, 0x01, 0x00, 0xA3, 0x0A, 0x00, 0x00, 0x00, 0x00,
	0x00, 0xC2, 0x00, 0x23, 0xC6, 0xE0, 0x98, 0x41, 0x00, 0x02, 0x83, 0x08, 0x00, 0x00, 0x00, 0x00,
	0x01, 0xB3, 0x09, 0xC1, 0x86, 0x18, 0xA5, 0x50, 0x01, 0x00, 0xA3, 0x0A, 0x00, 0x00, 0x00, 0x00,
	0x00, 0xC2, 0x00, 0x23, 0xC6, 0xE0, 0x98, 0x41, 0x00, 0x02, 0x03, 0x08, 0x00, 0x00, 0x00, 0x00,
	0x00, 0xC2, 0x00, 0x23, 0xC6, 0xE0, 0x98, 0x41, 0x00, 0x02, 0x05, 0x08, 0x00, 0x00, 0x00, 0x00,
	0x04, 0xC2, 0x0C, 0x23, 0xC5, 0xE0, 0xF6, 0x41, 0x00, 0x02, 0x05, 0x09, 0x00, 0x00, 0x00, 0x00,
	0x01, 0xC2, 0x00, 0x23, 0xC6, 0xE0, 0x98, 0x41, 0x00, 0x02, 0x05, 0x08, 0x00, 0x00, 0x00, 0x00,
	0x01, 0xC2, 0x82, 0x23, 0xF6, 0xE0, 0xD5, 0x41, 0x01, 0x02, 0x83, 0x0A, 0x00, 0x00, 0x00, 0x00,
	0x03, 0xBF, 0x09, 0xFF, 0xE3, 0xD0, 0x97, 0x50, 0x00, 0x00, 0xBB, 0x0A, 0x00, 0x00, 0x00, 0x00,
	0x0E, 0xBF, 0x07, 0xFF, 0xB5, 0xD1, 0x15, 0x50, 0x01, 0x00, 0xBB, 0x0A, 0x00, 0x00, 0x00, 0x00,
	0x01, 0xBF, 0x07, 0xC1, 0x77, 0xD1, 0x73, 0x50, 0x01, 0x00, 0xBB, 0x0A, 0x00, 0x00, 0x00, 0x00,
	0x0E, 0xF1, 0xC7, 0x38, 0x95, 0xAD, 0x78, 0x8E, 0x00, 0x02, 0x02, 0x09, 0x00, 0x00, 0x00, 0x00,
	0x01, 0xBF, 0x00, 0xFF, 0xF8, 0xD2, 0xB6, 0x50, 0x01, 0x00, 0xBA, 0x0A, 0x00, 0x00, 0x00, 0x00,
	0x0A, 0xC2, 0xC7, 0x23, 0x95, 0xE0, 0x78, 0x41, 0x00, 0x02, 0x7C, 0x09, 0x00, 0x00, 0x00, 0x00,
	0x01, 0xBF, 0x07, 0xC1, 0xF9, 0xD4, 0xB5, 0x50, 0x00, 0x00, 0xBB, 0x09, 0x00, 0x00, 0x00, 0x00,
	0xD1, 0xC2, 0x05, 0x23, 0xE7, 0xE0, 0x65, 0x41, 0x01, 0x02, 0x9D, 0x09, 0x00, 0x00, 0x00, 0x00,
	0x01, 0xFE, 0x00, 0x38, 0xE7, 0xA9, 0x94, 0x82, 0x00, 0x02, 0x03, 0x08, 0x00, 0x00, 0x00, 0x00,
	0x01, 0xBF, 0x00, 0xFF, 0xE7, 0xD8, 0x94, 0x50, 0x00, 0x00, 0xBB, 0x08, 0x00, 0x00, 0x00, 0x00,
	0x01, 0xBF, 0x00, 0xFF, 0x96, 0xD8, 0x67, 0x50, 0x00, 0x00, 0xBA, 0x08, 0x00, 0x00, 0x00, 0x00,
	0x01, 0xBF, 0x00, 0xFF, 0xB4, 0xDA, 0x26, 0x50, 0x00, 0x00, 0xBA, 0x08, 0x00, 0x00, 0x00, 0x00,
	0x01, 0xBF, 0x00, 0xC1, 0xB4, 0xDB, 0x26, 0x50, 0x00, 0x00, 0xBA, 0x08, 0x00, 0x00, 0x00, 0x00,
	0x95, 0x13, 0x81, 0x00, 0xE7, 0x95, 0x01, 0x65, 0x00, 0x00, 0x0E, 0x06, 0x00, 0x00, 0x00, 0x00,
	0x95, 0x13, 0x81, 0x00, 0xE7, 0x95, 0x01, 0x65, 0x00, 0x00, 0x0E, 0x06, 0x00, 0x00, 0x00, 0x00,
	0x10, 0xBF, 0x00, 0xC1, 0x96, 0xDE, 0x67, 0x50, 0x00, 0x00, 0xBA, 0x09, 0x00, 0x00, 0x00, 0x00,
	0x11, 0xBF, 0x00, 0xFF, 0x96, 0xDF, 0x67, 0x50, 0x00, 0x00, 0xBA, 0x09, 0x00, 0x00, 0x00, 0x00,
	0x00, 0xBF, 0x0E, 0xC1, 0x58, 0xD0, 0xDC, 0x50, 0x02, 0x00, 0xBA, 0x07, 0x00, 0x00, 0x00, 0x00,
	0x00, 0xBF, 0x0E, 0xFF, 0x5A, 0xD2, 0xD6, 0x50, 0x02, 0x00, 0xBA, 0x07, 0x00, 0x00, 0x00, 0x00,
	0x52, 0xBF, 0x07, 0xC1, 0x49, 0xD3, 0x04, 0x50, 0x03, 0x00, 0xBB, 0x08, 0x00, 0x00, 0x00, 0x00,
	0x52, 0xBF, 0x07, 0xC1, 0x41, 0xD4, 0x02, 0x50, 0x03, 0x00, 0xBB, 0x08, 0x00, 0x00, 0x00, 0x00,
	0x00, 0xBF, 0x0E, 0xFF, 0x5A, 0xD5, 0xD6, 0x50, 0x01, 0x00, 0xBA, 0x07, 0x00, 0x00, 0x00, 0x00,
	0x10, 0xBF, 0x0E, 0xC1, 0x53, 0xD6, 0x9F, 0x50, 0x01, 0x00, 0xBA, 0x07, 0x00, 0x00, 0x00, 0x00,
	0x11, 0xFE, 0x00, 0x38, 0xF5, 0xA9, 0x75, 0x80, 0x00, 0x02, 0x02, 0x08, 0x00, 0x00, 0x00, 0x00,
	0x04, 0xC2, 0x00, 0x23, 0xF8, 0xE0, 0xB6, 0x41, 0x01, 0x02, 0x03, 0x08, 0x00, 0x00, 0x00, 0x00,
	0x04, 0xC2, 0x00, 0x23, 0xF8, 0xE0, 0xB7, 0x41, 0x01, 0x02, 0x03, 0x08, 0x00, 0x00, 0x00, 0x00,
	0x01, 0xBF, 0x0B, 0xC1, 0x5E, 0xD8, 0xDC, 0x50, 0x01, 0x00, 0xBA, 0x07, 0x00, 0x00, 0x00, 0x00,
	0x00, 0xBF, 0x07, 0xC1, 0x5C, 0xDA, 0xDC, 0x50, 0x01, 0x00, 0xBA, 0x07, 0x00, 0x00, 0x00, 0x00,
	0xC5, 0xD5, 0x4F, 0x00, 0xF2, 0xF4, 0x60, 0x7A, 0x00, 0x00, 0x08, 0x06, 0x00, 0x00, 0x00, 0x00,
	0xC5, 0xD5, 0x4F, 0x00, 0xF2, 0xF2, 0x60, 0x72, 0x00, 0x00, 0x08, 0x06, 0x00, 0x00, 0x00, 0x00,
} ;
